@@ -217,6 +217,21 @@ hipblasStatus_t hipblasLtMatmulDescCreate(hipblasLtMatmulDesc_t* matmulDesc,
 try
 {
     rocblaslt::Debug::Instance().markerStart("hipblasLtMatmulDescCreate");
+    char* override = std::getenv("HIPBLASLT_OVERRIDE_COMPUTE_TYPE_XF32");
+    if (override && (computeType == hipblasComputeType_t::HIPBLAS_COMPUTE_32F_FAST_TF32)
+        && (std::string(override) != "")) {
+        switch (std::stoi(std::string(override))) {
+            case 0:
+                computeType = hipblasComputeType_t::HIPBLAS_COMPUTE_32F;
+                break;
+            case 2:
+                computeType = hipblasComputeType_t::HIPBLAS_COMPUTE_32F_FAST_16BF;
+                break;
+            case 1:
+            default:
+                break;
+        }
+    }
     auto status = RocBlasLtStatusToHIPStatus(rocblaslt_matmul_desc_create(
         (rocblaslt_matmul_desc*)matmulDesc, (rocblaslt_compute_type)computeType, scaleType));
     rocblaslt::Debug::Instance().markerStop();
