@@ -27,8 +27,6 @@
 #include "gemm.hpp"
 #include "runtime_args_selection.hpp"
 
-#include <rocRoller/Parameters/Solution/StreamK.hpp>
-
 #include "utility.hpp"
 
 using namespace rocRoller;
@@ -327,12 +325,6 @@ std::shared_ptr<GemmKernel> genGemmKernel(std::shared_ptr<SolutionParameters> ge
     params->setWaveTilesPerWavefront(wavetilePerWavefrontM, wavetilePerWavefrontN);
 
     {
-        auto memoryTypeA = MemoryType::WAVE;
-        if(gemm->direct2LDSA)
-            memoryTypeA = MemoryType::WAVE_Direct2LDS;
-        else if(gemm->loadLDSA)
-            memoryTypeA = MemoryType::LDS;
-
         auto macTileA = KernelGraph::CoordinateGraph::MacroTile(
             {gemm->workgroupTile.m, gemm->workgroupTile.k},
             LayoutType::MATRIX_A,
@@ -340,7 +332,7 @@ std::shared_ptr<GemmKernel> genGemmKernel(std::shared_ptr<SolutionParameters> ge
              gemm->machineInstruction.n,
              gemm->machineInstruction.k,
              gemm->machineInstruction.b},
-            memoryTypeA);
+            GetMemoryType(gemm->loadPathA));
         params->setDimensionInfo(tagLoadA, macTileA);
     }
 
@@ -361,12 +353,6 @@ std::shared_ptr<GemmKernel> genGemmKernel(std::shared_ptr<SolutionParameters> ge
     }
 
     {
-        auto memoryTypeB = MemoryType::WAVE;
-        if(gemm->direct2LDSB)
-            memoryTypeB = MemoryType::WAVE_Direct2LDS;
-        else if(gemm->loadLDSB)
-            memoryTypeB = MemoryType::LDS;
-
         auto macTileB = KernelGraph::CoordinateGraph::MacroTile(
             {gemm->workgroupTile.k, gemm->workgroupTile.n},
             LayoutType::MATRIX_B,
@@ -374,7 +360,7 @@ std::shared_ptr<GemmKernel> genGemmKernel(std::shared_ptr<SolutionParameters> ge
              gemm->machineInstruction.n,
              gemm->machineInstruction.k,
              gemm->machineInstruction.b},
-            memoryTypeB);
+            GetMemoryType(gemm->loadPathB));
         params->setDimensionInfo(tagLoadB, macTileB);
     }
 
