@@ -55,36 +55,11 @@ namespace rocRoller
                 bool scaleA = false;
                 bool scaleB = false;
 
-                matA = call(a.matA, b.matA);
-                if(a.matA == nullptr && b.matA == nullptr)
-                {
-                    matA = true;
-                }
-
-                matB = call(a.matB, b.matB);
-                if(a.matB == nullptr && b.matB == nullptr)
-                {
-                    matB = true;
-                }
-
-                matC = call(a.matC, b.matC);
-                if(a.matC == nullptr && b.matC == nullptr)
-                {
-                    matC = true;
-                }
-
+                matA   = call(a.matA, b.matA);
+                matB   = call(a.matB, b.matB);
+                matC   = call(a.matC, b.matC);
                 scaleA = call(a.scaleA, b.scaleA);
-                if(a.scaleA == nullptr && b.scaleA == nullptr)
-                {
-                    scaleA = true;
-                }
-
                 scaleB = call(a.scaleB, b.scaleB);
-                if(a.scaleB == nullptr && b.scaleB == nullptr)
-                {
-                    scaleB = true;
-                }
-
                 return matA && matB && matC && scaleA && scaleB;
             }
 
@@ -113,24 +88,9 @@ namespace rocRoller
                 bool r1hs = false;
                 bool r2hs = false;
 
-                lhs = call(a.lhs, b.lhs);
-                if(a.lhs == nullptr && b.lhs == nullptr)
-                {
-                    lhs = true;
-                }
-
+                lhs  = call(a.lhs, b.lhs);
                 r1hs = call(a.r1hs, b.r1hs);
-                if(a.r1hs == nullptr && b.r1hs == nullptr)
-                {
-                    r1hs = true;
-                }
-
                 r2hs = call(a.r2hs, b.r2hs);
-
-                if(a.r2hs == nullptr && b.r2hs == nullptr)
-                {
-                    r2hs = true;
-                }
                 return lhs && r1hs && r2hs;
             }
 
@@ -141,27 +101,13 @@ namespace rocRoller
                 bool rhs = false;
 
                 lhs = call(a.lhs, b.lhs);
-                if(a.lhs == nullptr && b.lhs == nullptr)
-                {
-                    lhs = true;
-                }
-
                 rhs = call(a.rhs, b.rhs);
-                if(a.rhs == nullptr && b.rhs == nullptr)
-                {
-                    rhs = true;
-                }
-
                 return lhs && rhs;
             }
 
             template <CUnary T>
             bool operator()(T const& a, T const& b)
             {
-                if(a.arg == nullptr && b.arg == nullptr)
-                {
-                    return true;
-                }
                 return call(a.arg, b.arg);
             }
 
@@ -170,6 +116,30 @@ namespace rocRoller
                 return a == b;
             }
 
+            bool operator()(BitfieldCombine const& a, BitfieldCombine const& b)
+            {
+                bool lhs = false;
+                bool rhs = false;
+
+                bool srcOffset = (a.srcOffset == b.srcOffset);
+                bool dstOffset = (a.dstOffset == b.dstOffset);
+                bool width     = (a.width == b.width);
+                bool srcIsZero = (a.srcIsZero == b.srcIsZero);
+                bool dstIsZero = (a.dstIsZero == b.dstIsZero);
+
+                lhs = call(a.lhs, b.lhs);
+                rhs = call(a.rhs, b.rhs);
+                return lhs && rhs && srcOffset && dstOffset && width && srcIsZero && dstIsZero;
+            }
+
+            bool operator()(BitFieldExtract const& a, BitFieldExtract const& b)
+            {
+                bool offset = (a.offset == b.offset);
+                bool width  = (a.width == b.width);
+
+                return offset && width && call(a.arg, b.arg);
+            }
+
             bool operator()(CommandArgumentPtr const& a, CommandArgumentPtr const& b)
             {
                 // Need to be careful not to invoke the overloaded operators, we want to compare
@@ -268,36 +238,11 @@ namespace rocRoller
                 bool scaleA = false;
                 bool scaleB = false;
 
-                matA = call(a.matA, b.matA);
-                if(a.matA == nullptr && b.matA == nullptr)
-                {
-                    matA = true;
-                }
-
-                matB = call(a.matB, b.matB);
-                if(a.matB == nullptr && b.matB == nullptr)
-                {
-                    matB = true;
-                }
-
-                matC = call(a.matC, b.matC);
-                if(a.matC == nullptr && b.matC == nullptr)
-                {
-                    matC = true;
-                }
-
+                matA   = call(a.matA, b.matA);
+                matB   = call(a.matB, b.matB);
+                matC   = call(a.matC, b.matC);
                 scaleA = call(a.scaleA, b.scaleA);
-                if(a.scaleA == nullptr && b.scaleA == nullptr)
-                {
-                    scaleA = true;
-                }
-
                 scaleB = call(a.scaleB, b.scaleB);
-                if(a.scaleB == nullptr && b.scaleB == nullptr)
-                {
-                    scaleB = true;
-                }
-
                 return matA && matB && matC && scaleA && scaleB;
             }
 
@@ -326,24 +271,9 @@ namespace rocRoller
                 bool r1hs = false;
                 bool r2hs = false;
 
-                lhs = call(a.lhs, b.lhs);
-                if(a.lhs == nullptr && b.lhs == nullptr)
-                {
-                    lhs = true;
-                }
-
+                lhs  = call(a.lhs, b.lhs);
                 r1hs = call(a.r1hs, b.r1hs);
-                if(a.r1hs == nullptr && b.r1hs == nullptr)
-                {
-                    r1hs = true;
-                }
-
                 r2hs = call(a.r2hs, b.r2hs);
-
-                if(a.r2hs == nullptr && b.r2hs == nullptr)
-                {
-                    r2hs = true;
-                }
                 return lhs && r1hs && r2hs;
             }
 
@@ -353,35 +283,15 @@ namespace rocRoller
                 bool lhs = false;
                 bool rhs = false;
 
-                lhs = call(a.lhs, b.lhs);
-                if(a.lhs == nullptr && b.lhs == nullptr)
-                {
-                    lhs = true;
-                }
-
-                rhs = call(a.rhs, b.rhs);
-                if(a.rhs == nullptr && b.rhs == nullptr)
-                {
-                    rhs = true;
-                }
-
+                lhs         = call(a.lhs, b.lhs);
+                rhs         = call(a.rhs, b.rhs);
                 bool result = lhs && rhs;
 
                 // Test if equivalent if expression is commutative
                 if(!result && CCommutativeBinary<T> && m_properties[AlgebraicProperty::Commutative])
                 {
-                    lhs = call(a.lhs, b.rhs);
-                    if(a.lhs == nullptr && b.rhs == nullptr)
-                    {
-                        lhs = true;
-                    }
-
-                    rhs = call(a.rhs, b.lhs);
-                    if(a.rhs == nullptr && b.lhs == nullptr)
-                    {
-                        rhs = true;
-                    }
-
+                    lhs    = call(a.lhs, b.rhs);
+                    rhs    = call(a.rhs, b.lhs);
                     result = lhs && rhs;
                 }
 
@@ -391,10 +301,6 @@ namespace rocRoller
             template <CUnary T>
             bool operator()(T const& a, T const& b)
             {
-                if(a.arg == nullptr && b.arg == nullptr)
-                {
-                    return true;
-                }
                 return call(a.arg, b.arg);
             }
 
@@ -408,6 +314,30 @@ namespace rocRoller
                 return (*a) == (*b);
             }
 
+            bool operator()(BitfieldCombine const& a, BitfieldCombine const& b)
+            {
+                bool lhs = false;
+                bool rhs = false;
+
+                bool srcOffset = (a.srcOffset == b.srcOffset);
+                bool dstOffset = (a.dstOffset == b.dstOffset);
+                bool width     = (a.width == b.width);
+                bool srcIsZero = (a.srcIsZero == b.srcIsZero);
+                bool dstIsZero = (a.dstIsZero == b.dstIsZero);
+
+                lhs = call(a.lhs, b.lhs);
+                rhs = call(a.rhs, b.rhs);
+                return lhs && rhs && srcOffset && dstOffset && width && srcIsZero && dstIsZero;
+            }
+
+            bool operator()(BitFieldExtract const& a, BitFieldExtract const& b)
+            {
+                bool offset = (a.offset == b.offset);
+                bool width  = (a.width == b.width);
+
+                return call(a.arg, b.arg) && offset && width;
+            }
+
             bool operator()(AssemblyKernelArgumentPtr const& a, AssemblyKernelArgumentPtr const& b)
             {
                 if(a->name == b->name)
@@ -559,7 +489,7 @@ namespace rocRoller
             bool includeRegisterComments = true;
 
             template <typename Expr>
-            requires(CUnary<Expr> || CBinary<Expr> || CTernary<Expr>) std::string
+            requires(CUnary<Expr> || CBinary<Expr> || CTernary<Expr> || CNary<Expr>) std::string
                 operator()(Expr const& expr) const
             {
                 return expr.comment;
