@@ -196,32 +196,12 @@ namespace rocRoller
                 = graph.coordinates.getNode<MacroTile>(graph.mapper.get<MacroTile>(tag));
             AssertFatal(existingMacTile.subTileSizes.size() == 4, "Invalid tile specification");
 
-            auto existingUnroll0 = graph.mapper.get<Unroll>(tag, 0);
-            auto existingUnroll1 = graph.mapper.get<Unroll>(tag, 1);
-            auto existingUnroll2 = graph.mapper.get<Unroll>(tag, 2);
-
-            // if unroll2 is -1, this returns 1.
-            int unrollKSize = getUnrollSize(graph, existingUnroll2);
-
-            int macKUnrollSize;
-            if(arg == NaryArgument::LHS_SCALE)
-            {
-                AssertFatal(existingUnroll1 != -1);
-                macKUnrollSize = getUnrollSize(graph, existingUnroll1);
-            }
-            else
-            {
-                AssertFatal(existingUnroll0 != -1);
-                macKUnrollSize = getUnrollSize(graph, existingUnroll0);
-            }
-
             // create new macrotile
-            auto macTile = MacroTile(
-                existingMacTile.sizes,
-                existingMacTile.layoutType,
-                {64, 64, existingMacTile.subTileSizes[2] * macKUnrollSize * unrollKSize, 1},
-                MemoryType::WAVE_SWIZZLE,
-                existingMacTile.subTileSizes);
+            auto macTile    = MacroTile(existingMacTile.sizes,
+                                     existingMacTile.layoutType,
+                                     existingMacTile.swizzleTileSizes,
+                                     MemoryType::WAVE_SWIZZLE,
+                                     existingMacTile.subTileSizes);
             auto macTileTag = graph.coordinates.addElement(macTile);
             connections.push_back(DC<MacroTile>(macTileTag));
 
@@ -372,6 +352,10 @@ namespace rocRoller
 
             std::map<int, int> unrolls;
 
+            auto existingUnroll0 = graph.mapper.get<Unroll>(tag, 0);
+            auto existingUnroll1 = graph.mapper.get<Unroll>(tag, 1);
+            auto existingUnroll2 = graph.mapper.get<Unroll>(tag, 2);
+
             if(arg == NaryArgument::LHS_SCALE)
             {
                 graph.coordinates.addElement(Tile(), {iWave0}, {SIMDBlock, SIMDIndex, laneInSIMD});
@@ -386,8 +370,7 @@ namespace rocRoller
                     graph.coordinates.addElement(Tile(), {nWave0}, {wave0, jammedWaveTile0});
                     connections.push_back(DC<JammedWaveTileNumber>(jammedWaveTile0, 0));
 
-                    auto unroll0 = graph.coordinates.addElement(
-                        Unroll(getUnrollSize(graph, existingUnroll0) / factor));
+                    auto unroll0 = existingUnroll0;
                     graph.coordinates.addElement(PassThrough(), {jammedWaveTile0}, {unroll0});
                     connections.push_back(DC<Unroll>(unroll0, 0));
                     unrolls[existingUnroll0] = unroll0;
@@ -396,8 +379,7 @@ namespace rocRoller
                 {
                     auto factor = macTile.subTileSizes.at(2) / existingMacTile.subTileSizes.at(2);
 
-                    auto unroll1 = graph.coordinates.addElement(
-                        Unroll(getUnrollSize(graph, existingUnroll1) / factor));
+                    auto unroll1 = existingUnroll1;
                     graph.coordinates.addElement(PassThrough(), {nWave1}, {unroll1});
                     connections.push_back(DC<Unroll>(unroll1, 1));
                     unrolls[existingUnroll1] = unroll1;
@@ -422,8 +404,7 @@ namespace rocRoller
                     graph.coordinates.addElement(Tile(), {nWave1}, {wave1, jammedWaveTile1});
                     connections.push_back(DC<JammedWaveTileNumber>(jammedWaveTile1, 1));
 
-                    auto unroll1 = graph.coordinates.addElement(
-                        Unroll(getUnrollSize(graph, existingUnroll1) / factor));
+                    auto unroll1 = existingUnroll1;
                     graph.coordinates.addElement(PassThrough(), {jammedWaveTile1}, {unroll1});
                     connections.push_back(DC<Unroll>(unroll1, 1));
                     unrolls[existingUnroll1] = unroll1;
@@ -432,8 +413,7 @@ namespace rocRoller
                 {
                     auto factor = macTile.subTileSizes.at(2) / existingMacTile.subTileSizes.at(2);
 
-                    auto unroll0 = graph.coordinates.addElement(
-                        Unroll(getUnrollSize(graph, existingUnroll0) / factor));
+                    auto unroll0 = existingUnroll0;
                     graph.coordinates.addElement(PassThrough(), {nWave0}, {unroll0});
                     connections.push_back(DC<Unroll>(unroll0, 0));
                     unrolls[existingUnroll0] = unroll0;
@@ -449,14 +429,36 @@ namespace rocRoller
             return {connections, exchangeConnections, unrolls};
         }
 
-        std::pair<int, int> getMergeFactors(KernelGraph const& graph, int macTileTag)
+        std::pair<int, int> getOuterMergeFactors(KernelGraph const& graph, int macTileTag)
         {
             auto macTile = graph.coordinates.getNode<MacroTile>(macTileTag);
             auto waveM   = macTile.subTileSizes.at(0);
             auto waveN   = macTile.subTileSizes.at(1);
             auto waveK   = macTile.subTileSizes.at(2);
 
-            AssertFatal(waveM == waveN, "waveM is not equal to waveN");
+            AssertFatal(
+                waveM == waveN, "waveM is not equal to waveN", ShowValue(waveM), ShowValue(waveN));
+
+            auto waveSwizzleM = macTile.swizzleTileSizes.at(0);
+            auto waveSwizzleN = macTile.swizzleTileSizes.at(1);
+            auto waveSwizzleK = macTile.swizzleTileSizes.at(2);
+
+            AssertFatal(waveSwizzleM == waveSwizzleN,
+                        "waveSwizzleM is not equal to waveSwizzleN",
+                        ShowValue(waveM),
+                        ShowValue(waveN));
+            return std::make_pair(waveSwizzleM / waveM, waveSwizzleK / waveK);
+        }
+
+        std::pair<int, int> getInnerMergeFactors(KernelGraph const& graph, int macTileTag)
+        {
+            auto macTile = graph.coordinates.getNode<MacroTile>(macTileTag);
+            auto waveM   = macTile.subTileSizes.at(0);
+            auto waveN   = macTile.subTileSizes.at(1);
+            auto waveK   = macTile.subTileSizes.at(2);
+
+            AssertFatal(
+                waveM == waveN, "waveM is not equal to waveN", ShowValue(waveM), ShowValue(waveN));
 
             auto const waveSwizzleMN = 64;
             auto const waveSwizzleK  = 4;
@@ -472,9 +474,6 @@ namespace rocRoller
         {
             AssertFatal(!scaleLoads.empty() && !loadUnrollMap.empty());
 
-            auto sampleTile          = scaleLoads.begin()->second;
-            auto [factorMN, factorK] = getMergeFactors(graph, sampleTile);
-
             std::map<int, std::vector<std::pair<int, int>>> mergeables;
 
             auto mergeLoadsByUnroll = [&](int fastDim,
@@ -561,6 +560,15 @@ namespace rocRoller
             // if unroll2 is -1, this returns 1.
             auto unrollKSize = getUnrollSize(graph, unroll2);
 
+            auto sampleTile                    = scaleLoads.begin()->second;
+            auto [innerFactorMN, innerFactorK] = getInnerMergeFactors(graph, sampleTile);
+            auto [outerFactorMN, outerFactorK] = getOuterMergeFactors(graph, sampleTile);
+            AssertFatal(
+                innerFactorMN == outerFactorMN, ShowValue(innerFactorMN), ShowValue(outerFactorMN));
+            AssertFatal(outerFactorMN % innerFactorMN == 0,
+                        ShowValue(innerFactorMN),
+                        ShowValue(outerFactorMN));
+
             if(arg == NaryArgument::LHS_SCALE)
             {
                 // A : M x K
@@ -568,16 +576,26 @@ namespace rocRoller
                 auto macKUnrollSize = getUnrollSize(graph, unroll1);
 
                 // merge scale loads
-                if(xUnrollSize % factorMN == 0 && macKUnrollSize % factorK == 0)
+                if(xUnrollSize % innerFactorMN == 0 && macKUnrollSize % innerFactorK == 0)
                 {
-                    mergeLoadsByUnroll(unroll0, unroll1, unroll2, factorMN, factorMN);
-                    mergeLoadsByUnroll(unroll1, unroll0, unroll2, factorK, macKUnrollSize);
-                    mergeLoadsByUnroll(unroll2,
-                                       unroll1,
-                                       unroll0,
-                                       unrollKSize,
-                                       unrollKSize,
-                                       macKUnrollSize / factorK);
+                    AssertFatal((macKUnrollSize * unrollKSize) % outerFactorK == 0,
+                                ShowValue(macKUnrollSize),
+                                ShowValue(unrollKSize),
+                                ShowValue(outerFactorK));
+
+                    mergeLoadsByUnroll(unroll0, unroll1, unroll2, innerFactorMN, outerFactorMN);
+                    if(macKUnrollSize % outerFactorK == 0)
+                        mergeLoadsByUnroll(unroll1, unroll0, unroll2, innerFactorK, outerFactorK);
+                    else
+                    {
+                        mergeLoadsByUnroll(unroll1, unroll0, unroll2, innerFactorK, macKUnrollSize);
+                        mergeLoadsByUnroll(unroll2,
+                                           unroll1,
+                                           unroll0,
+                                           outerFactorK / macKUnrollSize,
+                                           outerFactorK / macKUnrollSize,
+                                           macKUnrollSize / innerFactorK);
+                    }
                 }
             }
             if(arg == NaryArgument::RHS_SCALE)
@@ -587,16 +605,26 @@ namespace rocRoller
                 auto macKUnrollSize = getUnrollSize(graph, unroll0);
 
                 // merge scale loads
-                if(yUnrollSize % factorMN == 0 && macKUnrollSize % factorK == 0)
+                if(yUnrollSize % innerFactorMN == 0 && macKUnrollSize % innerFactorK == 0)
                 {
-                    mergeLoadsByUnroll(unroll1, unroll0, unroll2, factorMN, factorMN);
-                    mergeLoadsByUnroll(unroll0, unroll1, unroll2, factorK, macKUnrollSize);
-                    mergeLoadsByUnroll(unroll2,
-                                       unroll0,
-                                       unroll1,
-                                       unrollKSize,
-                                       unrollKSize,
-                                       macKUnrollSize / factorK);
+                    AssertFatal((macKUnrollSize * unrollKSize) % outerFactorK == 0,
+                                ShowValue(macKUnrollSize),
+                                ShowValue(unrollKSize),
+                                ShowValue(outerFactorK));
+
+                    mergeLoadsByUnroll(unroll1, unroll0, unroll2, innerFactorMN, outerFactorMN);
+                    if(macKUnrollSize % outerFactorK == 0)
+                        mergeLoadsByUnroll(unroll0, unroll1, unroll2, innerFactorK, outerFactorK);
+                    else
+                    {
+                        mergeLoadsByUnroll(unroll0, unroll1, unroll2, innerFactorK, macKUnrollSize);
+                        mergeLoadsByUnroll(unroll2,
+                                           unroll0,
+                                           unroll1,
+                                           outerFactorK / macKUnrollSize,
+                                           outerFactorK / macKUnrollSize,
+                                           macKUnrollSize / innerFactorK);
+                    }
                 }
             }
 
@@ -613,10 +641,6 @@ namespace rocRoller
                 return;
             }
 
-            auto sampleLoad     = scaleLoads.begin()->first;
-            auto unrollK        = graph.mapper.get<Unroll>(sampleLoad, 2);
-            auto forKUnrollSize = getUnrollSize(graph, unrollK);
-
             auto colouring = colourByUnrollValue(graph);
 
             auto loadUnrollMap = filterLoadUnrollColouring(colouring, scaleLoads);
@@ -628,7 +652,7 @@ namespace rocRoller
             if(mergeables.empty())
                 return;
 
-            sampleLoad = mergeables.begin()->first;
+            auto sampleLoad = mergeables.begin()->first;
             auto [loadConnections, exchangeConnections, unrollReindexMap]
                 = addSwizzleLoadCT(graph, context, sampleLoad, arg);
 
